clear all

set more off

   program define sem4gmm
      syntax varlist if, at(name) obsvar(varlist)
      // varlist is the list of vectorized residuals
      // at is the name of the parameter vector

      // create a variable to address the `if' condition
      marksample touse, novarlist

      // generate the implied matrix
      tempname Sigma
      mata : ParsToSigma( "`at'", "`Sigma'" )

      // produce vectorized deviations
      unab thevars : `obsvar'
      mata : VechData("`thevars'", "`varlist'", 1, "`touse'", st_matrix("`Sigma'") )
   end

   mata:

   void VechData( string scalar thevars, string scalar vechlist, | ///
      real scalar dev, string scalar tousename, real matrix Sigma) {

      // number of observed variables
      p = length( tokens( thevars ) )

      // center the variables before vech?
      if (missing(dev)) dev = 0;

      // subtract the supplied matrix?
      if (rows(Sigma)==0 | rows(Sigma)!=p ) Sigma = J( p, p, 0);

      // retrieve the data
      st_view(x=.,.,tokens(thevars), tousename )
      m = mean( x,1):/rows(x):*dev

      // retrieve the variables for vectorization
      st_view(v=.,.,tokens(vechlist), tousename )

      // compute vectorization observation by observation
      for( i=1;i<=rows(x); i++) {
         v[i,] = vech( (x[i,]-m)'*(x[i,]-m ) )' - vech(Sigma)'
      }

   }

   void ParsToSigma(string scalar atName, string scalar SigmaName) {

      real vector at, VarZeta;
      real matrix Lambda, Phi, Beta, Psi, CovXX, CovXY, CovYY;

      // obtain the vector of parameters from Stata vector `at'
      at = st_matrix( atName )

      // check the number of parameters
      if (cols(at)!=69) {
         st_matrix( SigmaName, J(0,0,.) )
         return
      }

      // set up the parameter matrices
      // loadings:
      Lambda = (1, at[1..16])'

      // covariances of the exogenous variables
      Phi = invvech( at[17..44]' )

      // variance of the latent regression term
      VarZeta = at[45]

      // regression coefficients
      Gamma = at[46..52]

      // measurement errors
      Psi  = diag( at[53..69] )

      // covariance of the exogenous variables goes as is
      CovXX = Phi

      // covariance between the explanatory variables and indicators
      CovXY = Phi * Gamma' * Lambda'

      // covariance of the indicators
      CovYY = Lambda * Gamma * Phi * Gamma' * Lambda' + ///
              Lambda * VarZeta * Lambda' + Psi

      // return the matrix back to Stata
      st_matrix(SigmaName, (CovXX, CovXY \ CovXY', CovYY) )
   }
   
   printf("{inp}.")
   end
   // of mata piece

use nhanes-adl-irt-bmx, clear
drop pfq061a pfq061k pfq061t
recode pfq* bpq* bmx* (5 =.a) (7=.b) (9=.c)
local k=0
foreach x of varlist pfq061* {
   local ++k
   gen byte bpfq`k' = (`x'>1) if !missing( `x' )
   local pfqlab `pfqlab' `k' "`x'"
   ren `x' pfq`k'
}
drop pfq0*
mkspline age1 40 age2 60 age3 70 age4 = ridageyr

gen byte highbp = (bpq020==1) & !mi( bpq020 )
gen byte female = riagendr - 1

// explanatory variables
local exovar age1 age2 age3 age4 female bmxbmi highbp

// indicators
unab endovar : pfq*

// estimation sample
gen byte _touse = 1
markout _touse `exovar' `endovar'
drop if ~_touse

// define parameter names:
   // the first 16 are lambda2..lambda17
   forvalues k=2/`: word count `endovar'' {
      local parlist `parlist' lambda`k'
   }

   // the next 28 are exogenous covariances
   forvalues i=1/`: word count `exovar'' {
      forvalues j=`i'/`: word count `exovar'' {
	     local parlist `parlist' phi_`i'_`j'
	  }
   }
   
   // the next one is VarZeta
   local parlist `parlist' Vzeta
   
   // the next 7 are regression coefficients
   forvalues i=1/`: word count `exovar'' {
      local parlist `parlist' gamma`i'
   }
   
   // the next 17 are measurement errors
   forvalues k=1/`: word count `endovar'' {
      local parlist `parlist' psi`k'
   }

local gmmsemopt parameters(`parlist') nequations( 300 ) obsvar(`exovar' `endovar')

// stupid starting values
matrix from1 = J(1,`: word count `parlist'',1)


// ULS
gmm sem4gmm if _touse , `gmmsemopt' winitial( identity ) from( from1 ) onestep vce( unadj )
est store uls_homosked


exit
